use std::time::Duration;
use eyre::{OptionExt, Result};
use rsa::{RsaPrivateKey, RsaPublicKey};
use rsa::pkcs1::LineEnding;
use rsa::pkcs8::{EncodePrivateKey, EncodePublicKey};
use sqlx::{Pool, Postgres};
use sqlx::postgres::PgPoolOptions;

pub async fn init_db(dsn: &str) -> Result<Pool<Postgres>> {
    let pool = PgPoolOptions::new()
        .acquire_timeout(Duration::from_secs(2)) // Fail fast and don't hang
        .max_connections(100)
        .connect(dsn)
        .await?;

    init_instance(&pool)?;

    Ok(pool)
}

const PRIVATE_KEY_BITS: usize = 2048;

async fn init_instance(pool: &Pool<Postgres>) -> Result<()> {
    let mut tx = pool.begin().await?;

    // Check if we already have an instance configuration
    let row = sqlx::query!("select exists(select * from instance)").fetch_one(&mut *tx).await?;
    if row.exists.ok_or_eyre("exists was null")? {
        return Ok(());
    }

    // Generate public/private key
    let mut rng = rand::thread_rng();
    let priv_key = RsaPrivateKey::new(&mut rng, PRIVATE_KEY_BITS)?;
    let pub_key = RsaPublicKey::from(&priv_key);

    let priv_key_string = priv_key.to_pkcs8_pem(LineEnding::default())?;
    let pub_key_string = pub_key.to_public_key_pem(LineEnding::default())?;

    sqlx::query!("insert into instance (public_key, private_key) values ($1, $2)",
        priv_key_string.to_string(), pub_key_string).execute(&mut *tx).await?;

    tx.commit().await?;

    Ok(())
}